package gr.ilsp.fc.utils;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;
import java.util.Random;

import org.apache.commons.io.FileUtils;
import org.apache.hadoop.fs.FileStatus;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.log4j.Logger;


public class DirUtils {
	private static final Logger LOGGER = Logger.getLogger(DirUtils.class);
	//private static String fs1 = System.getProperty("file.separator");
	@SuppressWarnings("deprecation")
	public static void clearPreviousLoopDir(FileSystem fs, Path outputPath, int curLoop) {
		Path dir;
		Path crawldb;
		String dirName = "";
		int index = 0;
		try {
			FileStatus[] files = fs.listStatus(outputPath);
			for (FileStatus status: files){
				if (!status.isDir()) continue;
				dir = status.getPath();
				dirName = dir.getName();
				try {
					index = Integer.parseInt(dirName.split("-")[0]);
				} catch (NumberFormatException e){
					continue;
				}
				crawldb = new Path(dir,CrawlConfig.CRAWLDB_SUBDIR_NAME);
				if (index == curLoop-2 && fs.exists(crawldb)){
					fs.delete(crawldb);
					LOGGER.debug("Deleted: "+crawldb);
					return;
				}				
			}
		} catch (IOException e) {
			LOGGER.info(e.getMessage());
		}
	}
	
	public static File createTempDir() {
	    final String baseTempPath = System.getProperty("java.io.tmpdir");

	    File tempDir = new File(baseTempPath + File.separator + "ilsp-fc-tempDir" + getRandomInt());
	    if (tempDir.exists() == false) {
	        tempDir.mkdir();
	    }
	    tempDir.deleteOnExit();
	    return tempDir;
	}

	public static int getRandomInt () {
	    return 1 + new Random().nextInt();
	}
	
	/**
	 * Deletes all sub-directories of a parent directory provided as a string.
	 * Sub-directories whose name equals one of the elements in the
	 * notToBeDeleted list are not deleted. If notToBeDeleted is null, all
	 * subdirectories are deleted.
	 * 
	 * Usage:
	 * 
	 * <pre>
	 * {@code
	 * String testDir = "/tmp/testDir"; //removeSubDirs(testDir, null);
	 * //Removes all subdirs. removeSubDirs(testDir, Arrays.asList(new
	 * String[]{"xml", "hey", "yo"})); //Removes all subdirs but xml, hey and yo
	 * 
	 * </pre>
	 * 
	 * @param parentDir
	 * @param notToBeDeleted
	 * @throws IOException
	 */
	public static void removeSubDirs(String dirName, List<String> notToBeDeleted)
			throws IOException {
		File parentDir = new File(dirName);
		removeSubDirs(parentDir, notToBeDeleted);
	}

	/**
	 * Deletes all sub-directories of a parent directory. Sub-directories whose
	 * name equals one of the elements in the notToBeDeleted list are not
	 * deleted. If notToBeDeleted is null, all subdirectories are deleted.
	 * 
	 * Usage:
	 * 
	 * <pre>
	 * {@code
	 * File testDir = newFile("/tmp/testDir");
	 * //removeSubDirs(testDir, null); //Removes all subdirs. 
	 * removeSubDirs(testDir, Arrays.asList(new String[]{"xml", "hey", "yo"})); //Removes all subdirs but xml, hey and yo
	 * </pre>
	 * 
	 * @param parentDir
	 * @param notToBeDeleted
	 * @throws IOException
	 */
	public static void removeSubDirs(File parentDir, List<String> notToBeDeleted) /*throws IOException*/ {
		if (parentDir.isDirectory()) {
			List<File> files = Arrays.asList(parentDir.listFiles());
			for (File file : files) {
				LOGGER.debug ("Checking "+file);
				if (notToBeDeleted!=null && notToBeDeleted.contains(file.getName())) {
					LOGGER.debug ("Not deleting "+file);
					continue;
				}
				if (file.isDirectory()) {
					try {
						FileUtils.deleteDirectory(file);
					} catch (IOException e) {
						// TODO Auto-generated catch block
						//e.printStackTrace();
						LOGGER.info("Unable to delete run dir: "+ file);
						continue;
					}
				}
			}
		} else {
			LOGGER.warn(parentDir + " is not a directory.");
		}
	}

	//@SuppressWarnings("deprecation")
	public static void deleteLoopDirs(FileSystem fs, Path outputPath, List<String> notToBeDeleted) {
		Path crawldir, crawldb, crawlcontent, crawlclassif, crawlparse;
		try {
			FileStatus[] crawldirs = listStatus(fs, outputPath);
			for (FileStatus status : crawldirs) {
				if (/*!status.isDir()* ||*/ notToBeDeleted.contains(status.getPath().getName())){ 
					continue;
				}else{
					//System.out.println(status.getPath());
					crawldir =status.getPath();
					
					crawldb = new Path(crawldir,CrawlConfig.CRAWLDB_SUBDIR_NAME);
					FileStatus[] crawldirfile = listStatus(fs, crawldb);
					for (FileStatus statusfile : crawldirfile) {
						//System.out.println(statusfile.getPath());
						fs.delete(statusfile.getPath(),true);
					}
					fs.delete(crawldb,true);
					
					crawlcontent = new Path(crawldir,CrawlConfig.CONTENT_SUBDIR_NAME);
					crawldirfile = listStatus(fs, crawlcontent);
					for (FileStatus statusfile : crawldirfile) {
						//System.out.println(statusfile.getPath());
						fs.delete(statusfile.getPath(),true);
					}
					fs.delete(crawlcontent,true);
					
					crawlclassif = new Path(crawldir,CrawlConfig.CLASSIFIER_SUBDIR_NAME);
					crawldirfile = listStatus(fs, crawlclassif);
					for (FileStatus statusfile : crawldirfile) {
						//System.out.println(statusfile.getPath());
						fs.delete(statusfile.getPath(),true);
					}
					fs.delete(crawlclassif,true);
					
					crawlparse = new Path(crawldir,CrawlConfig.PARSE_SUBDIR_NAME);
					crawldirfile = listStatus(fs, crawlparse);
					for (FileStatus statusfile : crawldirfile) {
						//System.out.println(statusfile.getPath());
						fs.delete(statusfile.getPath(),true);
					}
					fs.delete(crawlparse,true);
					
					fs.delete(crawldir,true);
					/*fs.deleteOnExit(crawldb);
					fs.deleteOnExit(crawlcontent);
					fs.deleteOnExit(crawlclassif);
					fs.deleteOnExit(crawlparse);
					fs.deleteOnExit(crawldir);*/
				}
					
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	private static FileStatus[] listStatus(FileSystem fs, Path path) throws IOException {
	    FileStatus[] result = fs.listStatus(path);
	    if (result == null) {
	        result = new FileStatus[0];
	    }
	    
	    return result;
	}

	public static void createDir(File outDir) {
		if (!outDir.exists()) {
			LOGGER.debug("Creating directory: " + outDir);
		    boolean result = false;
		    try{
		    	outDir.mkdir();
		        result = true;
		    } catch(SecurityException se){
		    	LOGGER.warn("Cannot create dir: " + outDir);
		    	throw se;
		    }        
		    if(result) {    
		    	LOGGER.debug("Created " + outDir);  
		    }
		}

	}
	
}
